﻿using NetinaShop.Domain.Entities.ProductCategories;

namespace NetinaShop.Repository.Handlers.ProductCategories;

public class GetProductCategoriesQueryHandler : IRequestHandler<GetProductCategoriesQuery, List<ProductCategorySDto>>
{
    private readonly IRepositoryWrapper _repositoryWrapper;

    public GetProductCategoriesQueryHandler(IRepositoryWrapper repositoryWrapper)
    {
        _repositoryWrapper = repositoryWrapper;
    }
    public async Task<List<ProductCategorySDto>> Handle(GetProductCategoriesQuery request, CancellationToken cancellationToken)
    {
        List<ProductCategorySDto> groupCats;
        if (request.Page != null)
        {
            groupCats = await _repositoryWrapper.SetRepository<ProductCategory>()
                .TableNoTracking
                .OrderByDescending(c => c.CreatedAt)
                .Skip(request.Page.Value * 15).Take(15)
                .Select(CategoryMapper.ProjectToSDto)
                .ToListAsync(cancellationToken);
        }
        else
        {
            groupCats = await _repositoryWrapper.SetRepository<ProductCategory>()
                .TableNoTracking
                .Select(CategoryMapper.ProjectToSDto)
                .ToListAsync(cancellationToken);
        }

        if (request.SortByMain == null || !request.SortByMain.Value)
            return groupCats;

        foreach (var cat in groupCats)
            cat.Children = groupCats.Where(c => c.ParentId == cat.Id).ToList();

        return groupCats.Where(c => c.IsMain).ToList();

    }
}